/********************************************************************
 *
 * colormap.c
 *  - einfaches Beispiel fuer ein Programm mit eigener Colormap
 *
 * Quelle: BREDEX GmbH, Braunschweig
 *
 ********************************************************************/

/* Header-Dateien */
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* Vorwaerts-Deklarationen */
Colormap createColormap();


main (argc, argv)
int argc;
char** argv;
{
    Display* display;           /* Display */
    Colormap cmap;              /* eigene Colormap */
    XSetWindowAttributes wa;    /* Struktur fuer Fenster-Attribute */
    Window window;              /* Fenster mit eigener Colormap */
    XEvent event;               /* X-Event */

    /* Display oeffnen (Verbindung zum X-Server herstellen)
     */
    display = XOpenDisplay ((char*)NULL);
    if (display == NULL) {
        fprintf (stderr, "can't open \"%s\"\n", XDisplayName((char*)NULL));
        exit (1);
    }

    /* eigene Colormap erzeugen und mit einigen Farben fuellen
     */
    cmap = createColormap (display);

    /* Fenster mit eigener Colormap erzeugen und ausgeben
     */
    wa.colormap = cmap;                 /* eigene Colormap */
    wa.background_pixel = 3;            /* Farbe Weiss der eigenen Colormap */
    wa.event_mask = ButtonPressMask;    /* ButtonPress-Events selektiert */
    window = XCreateWindow (display, DefaultRootWindow(display),
                            0, 0, 200, 200, 8, CopyFromParent, InputOutput, 
                            (Visual*)CopyFromParent, 
                            CWColormap | CWBackPixel | CWEventMask, &wa);
    XSetStandardProperties (display, window, "Eigene Colormap", "Colormap",
                                (Pixmap)None, argv, argc, (XSizeHints*)NULL);
    XMapWindow (display, window);

    /* Ereignis-Schleife
     */
    for (;;) {
        XNextEvent (display, &event);

        /* Mausbutton beendet das Programm
         */
        if (event.type == ButtonPress) {
            exit (0);
        }
    }
}


/* createColormap
 *  - eigene Colormap erzeugen, mit einigen Farben fuellen
 *    und zurueckliefern
 */
Colormap createColormap (display)
Display* display;
{
    Colormap cmap;

    /* Colormap erzeugen (AllocAll: alle Farbzellen sind reserviert)
     */
    cmap = XCreateColormap (display, DefaultRootWindow(display),
                            DefaultVisual (display, DefaultScreen(display)),
                            AllocAll);

    /* Farbwerte 0 bis 5 mit eigenen Farben fuellen
     * (Weiss ist Farbwert 3)
     */
    XStoreNamedColor (display, cmap, "red",    0, DoRed|DoGreen|DoBlue);
    XStoreNamedColor (display, cmap, "green",  1, DoRed|DoGreen|DoBlue);
    XStoreNamedColor (display, cmap, "blue",   2, DoRed|DoGreen|DoBlue);
    XStoreNamedColor (display, cmap, "white",  3, DoRed|DoGreen|DoBlue);
    XStoreNamedColor (display, cmap, "yellow", 4, DoRed|DoGreen|DoBlue);
    XStoreNamedColor (display, cmap, "black",  5, DoRed|DoGreen|DoBlue);

    return cmap;
}
